//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Mask/MaskListModel.cpp
//! @brief     Implements class MaskListModel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Mask/MaskListModel.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Mask/MaskContainerItem.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/Model/Mask/ProjectionContainerItem.h"

// Implementation of MaskListModel is based on the Qt source code for QStringListModel

MaskListModel::MaskListModel(MaskContainerItem* item)
    : m_mask_container_item(item)
    , m_selection_model(new QItemSelectionModel(this))
{
}

MaskListModel::~MaskListModel() = default;

// Begin overridden methods from QAbstractListModel

int MaskListModel::rowCount(const QModelIndex& parent) const
{
    if (parent.isValid())
        return 0;

    return m_mask_container_item->size();
}

QVariant MaskListModel::data(const QModelIndex& index, int role) const
{
    const qsizetype row = index.row();

    if (row < 0 || row >= m_mask_container_item->size())
        return {};

    if (role == Qt::DisplayRole || role == Qt::EditRole)
        return {m_mask_container_item->at(row)->maskName()};

    return {};
}

// End overridden methods from QAbstractListModel

void MaskListModel::insertMask(int row, MaskItem* item)
{
    QAbstractListModel::beginInsertRows(m_mask_container_item->rootIndex, row, row);
    m_mask_container_item->insertMask(row, item);
    QAbstractListModel::endInsertRows();
}

//! Move mask to a given row
void MaskListModel::moveMask(int from_row, int to_row)
{
    emit QAbstractListModel::beginMoveRows(m_mask_container_item->rootIndex, from_row, from_row,
                                           m_mask_container_item->rootIndex, to_row);
    m_mask_container_item->moveMask(from_row, to_row);
    emit QAbstractListModel::endMoveRows();
}

void MaskListModel::removeMaskAt(int row)
{
    QAbstractListModel::beginRemoveRows(m_mask_container_item->rootIndex, row, row);
    m_mask_container_item->removeMaskAt(row);
    QAbstractListModel::endRemoveRows();
}

void MaskListModel::removeMask(MaskItem* item)
{
    const int row = m_mask_container_item->indexOfItem(item);
    removeMaskAt(row);
}

QModelIndex MaskListModel::indexOfItem(const MaskObject* item) const
{
    if (const auto* ii = dynamic_cast<const MaskItem*>(item)) {
        const int row = m_mask_container_item->indexOfItem(ii);
        return QAbstractListModel::index(row, 0, {});
    }
    return {};
}

MaskItem* MaskListModel::itemForIndex(const QModelIndex& index) const
{
    if (index.isValid())
        return m_mask_container_item->at(index.row());

    return nullptr;
}

void MaskListModel::clear()
{
    QAbstractListModel::beginResetModel();
    m_mask_container_item->clear();
    QAbstractListModel::endResetModel();
}

ProjectionContainerItem* MaskListModel::projnItem()
{
    return dynamic_cast<ProjectionContainerItem*>(m_mask_container_item);
}

const ProjectionContainerItem* MaskListModel::projnItem() const
{
    return dynamic_cast<const ProjectionContainerItem*>(m_mask_container_item);
}
